import * as ast from "./index";


/**
 * Interface for pseuCo AST visitors.
 */
export abstract class AbstractVisitor<T> {

    abstract visitProgram(node: ast.Program, context?: unknown): T;

    abstract visitMainAgent(node: ast.MainAgent, context?: unknown): T;

    abstract visitProcedureDeclaration(node: ast.ProcedureDeclaration, context?: unknown): T;

    abstract visitProcedureParameter(node: ast.ProcedureParameter, context?: unknown): T;

    abstract visitMonitor(node: ast.Monitor, context?: unknown): T;

    abstract visitStruct(node: ast.Struct, context?: unknown): T;

    abstract visitConditionDeclaration(node: ast.ConditionDeclaration, context?: unknown): T;

    abstract visitVariableDeclarations(node: ast.VariableDeclarations, context?: unknown): T;

    abstract visitVariableDeclarator(node: ast.VariableDeclarator, context?: unknown): T;

    abstract visitVariableInitializer(node: ast.VariableInitializer, context?: unknown): T;

    abstract visitArrayTypeNode(node: ast.ArrayTypeNode, context?: unknown): T;

    abstract visitSimpleTypeNode(node: ast.SimpleTypeNode, context?: unknown): T;

    abstract visitChannelTypeNode(node: ast.ChannelTypeNode, context?: unknown): T;

    abstract visitClassTypeNode(node: ast.ClassTypeNode, context?: unknown): T;

    abstract visitStartExpression(node: ast.StartExpression, context?: unknown): T;

    abstract visitAssignExpression(node: ast.AssignExpression, context?: unknown): T;

    abstract visitAssignDestination(node: ast.AssignDestination, context?: unknown): T;

    abstract visitSendExpression(node: ast.SendExpression, context?: unknown): T;

    abstract visitConditionalExpression(node: ast.ConditionalExpression, context?: unknown): T;

    abstract visitOrExpression(node: ast.OrExpression, context?: unknown): T;

    abstract visitAndExpression(node: ast.AndExpression, context?: unknown): T;

    abstract visitEqualityExpression(node: ast.EqualityExpression, context?: unknown): T;

    abstract visitRelationalExpression(node: ast.RelationalExpression, context?: unknown): T;

    abstract visitAdditiveExpression(node: ast.AdditiveExpression, context?: unknown): T;

    abstract visitMultiplicativeExpression(node: ast.MultiplicativeExpression, context?: unknown): T;

    abstract visitUnaryExpression(node: ast.UnaryExpression, context?: unknown): T;

    abstract visitPostfixExpression(node: ast.PostfixExpression, context?: unknown): T;

    abstract visitReceiveExpression(node: ast.ReceiveExpression, context?: unknown): T;

    abstract visitProcedureCall(node: ast.ProcedureCall, context?: unknown): T;

    abstract visitClassCall(node: ast.ClassCall, context?: unknown): T;

    abstract visitArrayExpression(node: ast.ArrayExpression, context?: unknown): T;

    abstract visitLiteralExpression(node: ast.LiteralExpression, context?: unknown): T;

    abstract visitIdentifierExpression(node: ast.IdentifierExpression, context?: unknown): T;

    abstract visitStatementWrapper(node: ast.StatementWrapper, context?: unknown): T;

    abstract visitBreakStatement(node: ast.BreakStatement, context?: unknown): T;

    abstract visitContinueStatement(node: ast.ContinueStatement, context?: unknown): T;

    abstract visitStatementBlock(node: ast.StatementBlock, context?: unknown): T;

    abstract visitStatementExpression(node: ast.StatementExpression, context?: unknown): T;

    abstract visitSelectStatement(node: ast.SelectStatement, context?: unknown): T;

    abstract visitCase(node: ast.Case, context?: unknown): T;

    abstract visitConditionStatement(node: ast.ConditionStatement, context?: unknown): T;

    abstract visitIfStatement(node: ast.IfStatement, context?: unknown): T;

    abstract visitWhileStatement(node: ast.WhileStatement, context?: unknown): T;

    abstract visitDoStatement(node: ast.DoStatement, context?: unknown): T;

    abstract visitForStatement(node: ast.ForStatement, context?: unknown): T;

    abstract visitForInit(node: ast.ForInit, context?: unknown): T;

    abstract visitReturnStatement(node: ast.ReturnStatement, context?: unknown): T;

    abstract visitPrimitiveStatement(node: ast.PrimitiveStatement, context?: unknown): T;

    abstract visitPrintStatement(node: ast.PrintStatement, context?: unknown): T;
}