import { Node, AndExpression, ClassCall, ConditionalExpression, EqualityExpression, IdentifierExpression, LiteralExpression, OrExpression, ProcedureCall, ReceiveExpression, RelationalExpression, SendExpression, UnaryExpression } from "./index";
import { Visitor } from "./../Visitor";

export type BooleanExpression = AndExpression | ClassCall | ConditionalExpression | EqualityExpression | IdentifierExpression | LiteralExpression | OrExpression | ProcedureCall | ReceiveExpression | RelationalExpression | SendExpression | UnaryExpression;

/**
 * AST node representing condition declarations. Can only be declared on the top level of monitor declarations.
 * @example condition c with x > 2;
 */
export class ConditionDeclaration extends Node {
    readonly nodeName = "ConditionDeclaration";

    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param name The name of the declared condition.
     * @param expression The boolean expression to determine if the condition is satisfied.
     */
    constructor(line: number, column: number, readonly name: string, expression: BooleanExpression) {
        super(line, column, expression);
    }

    /**
     * The boolean expression to determine if the condition is satisfied.
     */
    get expression(): BooleanExpression {
        return this.childNodes[0] as BooleanExpression;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitConditionDeclaration(this, context);
    }

    override indentedString(indent: string): string {
        return `${indent}condition ${this.name} with ${this.expression.toString()};`;
    }


    // Backwards compatibility

    getExpression(): BooleanExpression {
        return this.expression;
    }
    
}