import { Node, StatementBlock } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing the main agent of a pseuCo program.
 * @example mainAgent {
 *   // this.body: StatementBlock
 * }
 */
export class MainAgent extends Node {
    readonly nodeName = "MainAgent";

    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param body The only child of this node is a `StatementBlock`.
     */
    constructor(line: number, column: number, body: StatementBlock) {
        super(line, column, body);
    }

    /**
     * The only child of this node is a `StatementBlock`.
     */
    get body(): StatementBlock {
        return this.childNodes[0] as StatementBlock;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitMainAgent(this, context);
    }

    override indentedString(indent: string): string {
        return `mainAgent ${this.body.indentedString(indent)}`;
    }

    override insideProcedure(): boolean {
        return true;
    }

}