import { Node, TypeNode } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing a parameter of a procedure.
 * @example int x
 */
export class ProcedureParameter extends Node {
    readonly nodeName = "ProcedureParameter";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param typeNode The AST node representing the type (e.g., `int`)
     * @param name The name of the parameter (e.g., `x`)
     */
    constructor(line: number, column: number, typeNode: TypeNode, readonly name: string) {
        super(line, column, typeNode);
    }

    /**
     * The AST node representing the type (e.g., `int`)
     */
    get typeNode(): TypeNode {
        return (this.childNodes[0] as TypeNode);
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitProcedureParameter(this, context);
    }

    override indentedString(_: string): string {
        return `${this.typeNode.toString()} ${this.name}`;
    }


    // Backwards compatibility

    /**
     * @deprecated
     */
    get identifier(): string {
        return this.name;
    }

    // getVariable()??
}