import { Node, Monitor, Struct, MainAgent, VariableDeclarations, ProcedureDeclaration } from "./index";
import { Visitor } from "./../Visitor";

export type ProgramDeclaration = Monitor | Struct | MainAgent | VariableDeclarations | ProcedureDeclaration;   

/**
 * Represents the root node of a pseuCo AST.
 */
export class Program extends Node {
    readonly nodeName = "Program";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param children The child nodes of a `Program` are declarations, i.e., monitor declarations, struct declarations, main agent declaration, variable declarations or procedure declarations.
     */
    constructor(line: number, column: number, ...children: ProgramDeclaration[]) {
        super(line, column, ...children);
    }

    /**
     * All declarations of the program, i.e., all monitor declarations, struct declarations, the main agent declaration, variable declarations, and procedure declarations.
     */
    get declarations(): ProgramDeclaration[] {
        return this.childNodes as ProgramDeclaration[];
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitProgram(this, context);
    }

    override indentedString(indent: string) : string {
        return this.childNodes.map(function(child) {
            return child.indentedString(indent);
        }).join("\n");
    }

}