import { Expression } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing an indexed access of an array.
 * @example a[i]
 */
export class ArrayExpression extends Expression {
    readonly nodeName = "ArrayExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param instanceExpression The AST node representing the reference to the array (e.g., `a`).
     * @param indexExpression The AST node representing the index of the element to access in the array (e.g., `i`).
     */
    constructor(line: number, column: number, instanceExpression: Expression, indexExpression: Expression) {
        super(line, column, instanceExpression, indexExpression);
    }

    /**
     * The AST node representing the reference to the array (e.g., `a`).
     */
    get instanceExpression(): Expression {
        return this.childNodes[0] as Expression;
    }

    /**
     * The AST node representing the index of the element to access in the array (e.g., `i`).
     */
    get indexExpression(): Expression {
        return this.childNodes[1] as Expression;
    }

    override get precedence(): number {
        return 81;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitArrayExpression(this, context);
    }

    override indentedString(indent: string): string {
        return `${this.instanceExpression.toString()}[${this.indexExpression.toString()}]`;
    }
}