import { Expression } from "./index";
import { Visitor } from "./../Visitor";
import { ProcedureCall } from "../typeNodes";

// a.b.c() is not allowed, only a.b().c() is possible. -> callExpression is always ProcedureCall

/**
 * AST node representing the call of a monitor or struct procedure.
 * @example m.f()
 */
export class ClassCall extends Expression {
    readonly nodeName = "ClassCall";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param instanceExpression AST node representing the expression that evaluates to a monitor or struct reference.
     * @param callExpression The AST node representing the procedure call expression.
     */
    constructor(line: number, column: number, instanceExpression: Expression, callExpression: ProcedureCall) {
        super(line, column, instanceExpression, callExpression);
    }

    /**
     * AST node representing the expression that evaluates to a monitor or struct reference.
     */
    get instanceExpression(): Expression {
        return this.childNodes[0] as Expression;
    }

    /**
     * The AST node representing the procedure call expression.
     */
    get callExpression(): ProcedureCall {
        return this.childNodes[1] as ProcedureCall;
    }

    override get precedence(): number {
        return 78;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitClassCall(this, context);
    }

    override indentedString(indent: string): string {
        return `${this.instanceExpression.toString()}.${this.callExpression.toString()}`;
    }
}