import { Expression } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing identifiers used as expressions.
 * @example x
 */
export class IdentifierExpression extends Expression {
    readonly nodeName = "IdentifierExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param identifier The identifier as a string.
     */
    constructor(line: number, column: number, readonly identifier: string) {
        super(line, column);
    }

    override get precedence(): number {
        return 84;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitIdentifierExpression(this, context);
    }

    override indentedString(_: string): string {
        return this.identifier;
    }
}