import { Expression } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing the disjunction of two values.
 * @example a || b
 */
export class OrExpression extends Expression {
    readonly nodeName = "OrExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param left The AST node representing the left operand.
     * @param right The AST node representing the right operand.
     */
    constructor(line: number, column: number, left: Expression, right: Expression) {
        super(line, column, left, right);
    }

    /**
     * The AST node representing the left operand.
     */
    get left(): Expression {
        return this.childNodes[0] as Expression;
    }

    /**
     * The AST node representing the right operand.
     */
    get right(): Expression {
        return this.childNodes[1] as Expression;
    }

    override get precedence(): number {
        return 48;
    }

    override indentedString(indent: string): string {
        return `${this.childToString(this.left)} || ${this.childToString(this.right, 1)}`;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitOrExpression(this, context);
    }
}