import { Expression, AssignDestination } from "./index";
import { Visitor } from "./../Visitor";

export type PostfixOperator = "++" | "--";

/**
 * AST node representing increment or decrement by one operations.
 * @example x++
 * x--
 */
export class PostfixExpression extends Expression {
    readonly nodeName = "PostfixExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param assignDestination AST node representing the value to increment or decrement.
     * @param operator The operator distinguishing between increment ("++") and decrement ("--").
     */
    constructor(line: number, column: number, assignDestination: AssignDestination, readonly operator: PostfixOperator) {
        super(line, column, assignDestination);
    }

    /**
     * AST node representing the value to increment or decrement.
     */
    get assignDestination(): AssignDestination {
        return (this.childNodes[0] as AssignDestination);
    }

    override get precedence(): number {
        return 69;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitPostfixExpression(this, context);
    }

    override indentedString(indent: string): string {
        return `${this.assignDestination.toString()}${this.operator}`;
    }
}