import { Expression, Node } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing receive expressions.
 * @example <? chan
 */
export class ReceiveExpression extends Expression {
    readonly nodeName = "ReceiveExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param channelExpression The AST node representing the expression that evaluates to the channel from which a value shall be received.
     */
    constructor(line: number, column: number, channelExpression: Expression) {
        super(line, column, channelExpression);
    }

    /**
     * The AST node representing the expression that evaluates to the channel from which a value shall be received.
     */
    get channelExpression(): Expression {
        return this.childNodes[0] as Expression;
    }

    override get precedence(): number {
        return 72;
    }

    override indentedString(indent: string): string {
        return `<? ${this.childToString(this.channelExpression)}`;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitReceiveExpression(this, context);
    }

    override usedSendOrReceiveOperators(): number {
        return this.childNodes.reduce(function(acc: number, child: Node) {
            return acc + child.usedSendOrReceiveOperators();
        }, 1);
    }
}