import { Expression, ProcedureCall, ClassCall } from "./index";
import { Visitor } from "./../Visitor";

export type CallExpression = ProcedureCall | ClassCall

/**
 * The AST node representing the start of a new agent.
 * @example start(someProcedure())
 */
export class StartExpression extends Expression {
    readonly nodeName = "StartExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param callExpression The procedure call with which the new agent shall be started. Can be a call of a global, monitor or struct procedure.
     */
    constructor(line: number, column: number, callExpression: CallExpression) {
        super(line, column, callExpression);
    }

    /**
     * The procedure call with which the new agent shall be started. Can be a call of a global, monitor or struct procedure.
     */
    get callExpression(): CallExpression {
        return (this.childNodes[0] as CallExpression);
    }

    override get precedence(): number {
        return 42;
    }

    

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitStartExpression(this, context);
    }

    override indentedString(indent: string): string {
        return `${indent}start (${this.callExpression.toString()})`;
    }

    override containsProcedureCall(): boolean {
        return true;
    }
}