import { Expression } from "./index";
import { Visitor } from "./../Visitor";

export type UnaryOperator = "!" | "-" | "+";

/**
 * AST node representing unary expressions.
 * @example -x
 * +x
 * !b
 */
export class UnaryExpression extends Expression {
    readonly nodeName = "UnaryExpression";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param operator The unary operator, i.e., "-", "+" or "!"
     * @param valueExpression The AST node representing the expression that evaluates to the value on which the operation shall be applied to.
     */
    constructor(line: number, column: number, readonly operator: UnaryOperator, valueExpression: Expression) {
        super(line, column, valueExpression);
    }

    /**
     * The AST node representing the expression that evaluates to the value on which the operation shall be applied to.
     */
    get valueExpression(): Expression {
        return this.childNodes[0] as Expression;
    }

    override get precedence(): number {
        return 66;
    }

    override indentedString(indent: string): string {
        return `${this.operator} ${this.childToString(this.valueExpression)}`;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitUnaryExpression(this, context);
    }
}