import { MonitorDeclaration, StructDeclaration } from "./declarations";

export { Node } from "./Node";

// Declarations
export { Program } from "./declarations/Program";
export { MainAgent } from "./declarations/MainAgent";
export { ConditionDeclaration } from "./declarations/ConditionDeclaration";
export { Monitor, MonitorDeclaration } from "./declarations/Monitor";
export { ProcedureDeclaration } from "./declarations/ProcedureDeclaration";
export { ProcedureParameter } from "./declarations/ProcedureParameter";
export { Struct, StructDeclaration } from "./declarations/Struct";
export { VariableDeclarations } from "./declarations/VariableDeclarations";
export { VariableDeclarator } from "./declarations/VariableDeclarator";
export { VariableInitializer } from "./declarations/VariableInitializer";
// Backwards compatibility:
/**
 * @deprecated
 */
export type ClassDeclaration = StructDeclaration | MonitorDeclaration;

// Expressions
export { Expression } from "./expressions/Expression";
export { AdditiveExpression, AdditiveOperator } from "./expressions/AdditiveExpression";
export { AndExpression } from "./expressions/AndExpression";
export { ArrayExpression } from "./expressions/ArrayExpression";
export { AssignExpression, AssignOperator } from "./expressions/AssignExpression";
export { AssignDestination } from "./expressions/AssignDestination";
export { ClassCall } from "./expressions/ClassCall";
export { ConditionalExpression } from "./expressions/ConditionalExpression";
export { EqualityExpression, EqualityOperator } from "./expressions/EqualityExpression";
export { IdentifierExpression } from "./expressions/IdentifierExpression";
export { LiteralExpression, ValueWrapper, ValueType } from "./expressions/LiteralExpression";
export { MultiplicativeExpression, MultiplicativeOperator } from "./expressions/MultiplicativeExpression";
export { OrExpression } from "./expressions/OrExpression";
export { PostfixExpression, PostfixOperator } from "./expressions/PostfixExpression";
export { ProcedureCall } from "./expressions/ProcedureCall";
export { ReceiveExpression } from "./expressions/ReceiveExpression";
export { RelationalExpression, RelationalOperator } from "./expressions/RelationalExpression";
export { SendExpression } from "./expressions/SendExpression";
export { StartExpression, CallExpression } from "./expressions/StartExpression";
export { UnaryExpression, UnaryOperator } from "./expressions/UnaryExpression";

// Type Nodes
export { TypeNode } from "./typeNodes/TypeNode";
export { ArrayTypeNode } from "./typeNodes/ArrayTypeNode";
export { ChannelTypeName, ChannelTypeNode } from "./typeNodes/ChannelTypeNode";
export { ClassTypeNode } from "./typeNodes/ClassTypeNode";
export { SimpleTypeName, SimpleTypeNode } from "./typeNodes/SimpleTypeNode";

// Statements
export { BreakStatement } from "./statements/BreakStatement";
export { Case } from "./statements/Case";
export { ConditionStatement } from "./statements/ConditionStatement";
export { ContinueStatement } from "./statements/ContinueStatement";
export { DoStatement } from "./statements/DoStatement";
export { ForInit } from "./statements/ForInit";
export { ForStatement } from "./statements/ForStatement";
export { IfStatement } from "./statements/IfStatement";
export { PrimitiveStatement, Primitive } from "./statements/PrimitiveStatement";
export { PrintStatement } from "./statements/PrintStatement";
export { ReturnStatement } from "./statements/ReturnStatement";
export { SelectStatement } from "./statements/SelectStatement";
export { StatementBlock } from "./statements/StatementBlock";
export { StatementExpression, StatementableExpression } from "./statements/StatementExpression";
export { StatementWrapper, Statement } from "./statements/StatementWrapper";
export { WhileStatement } from "./statements/WhileStatement";

// Visitors
export { AbstractVisitor } from "./AbstractVisitor";
export { Visitor } from "./Visitor";
export { VoidVisitor } from "./VoidVisitor";
