import { Node } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing the `continue` statement.
 * @example continue;
 */
export class ContinueStatement extends Node {
    readonly nodeName = "ContinueStatement";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     */
    constructor(line: number, column: number) {
        super(line, column);
    }

    override indentedString(indent: string): string {
        return `${indent}continue;`;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitContinueStatement(this, context);
    }
}