import { Node, StatementWrapper, Expression } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing a do-while loop.
 * @example do {
 *   // statements
 * } while(predicate);
 */
export class DoStatement extends Node {
    readonly nodeName = "DoStatement";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param loopBody The AST node representing the body of the loop.
     * @param predicate The AST node representing the boolean predicate expression.
     */
    constructor(line: number, column: number, loopBody: StatementWrapper, predicate: Expression) {
        super(line, column, loopBody, predicate);
    }

    /**
     * The AST node representing the boolean predicate expression.
     */
    get predicate(): Expression {
        return this.childNodes[1] as Expression;
    }

    /**
     * The AST node representing the body of the loop.
     */
    get loopBody(): StatementWrapper {
        return this.childNodes[0] as StatementWrapper;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitDoStatement(this, context);
    }

    override insideLoop(): boolean {
        return true;
    }

    override indentedString(indent: string): string {
        return `${indent}do ${this.loopBody.indentedString(indent)}\n${indent}while (${this.predicate.toString()});`;
    }
}