import { Node } from "./index";
import { Visitor } from "./../Visitor";
import { StatementExpression } from "./StatementExpression";
import { VariableDeclarations } from "../declarations/VariableDeclarations";

export type ForVariableInitialisation = StatementExpression | VariableDeclarations

/**
 * The AST node representing the initialisation of a `for` loop.
 */
export class ForInit extends Node {
    readonly nodeName = "ForInit";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param initialisations An array of AST nodes of [StatementExpressions](./StatementExpression.ts) or variable declarations.
     */
    constructor(line: number, column: number, ...initialisations: ForVariableInitialisation[]) {
        super(line, column, ...initialisations);
    }

    /**
     * An array of AST nodes of [StatementExpressions](./StatementExpression.ts) or variable declarations.
     */
    get variableInitialisations(): ForVariableInitialisation[] {
        return (this.childNodes as ForVariableInitialisation[]);
    }

    override indentedString(indent: string): string {
        const initsString = this.variableInitialisations.map(function(init) {
            return init.toString();
        }).join(", ");
        return initsString;
    }
    

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitForInit(this, context);
    }
}