import { Node, TypeNode } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing an array type.
 * @example int[4]
 */
export class ArrayTypeNode extends TypeNode {
    readonly nodeName = "ArrayTypeNode";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param baseType AST node representing the type of the elements in the array.
     * @param size The size of the array.
     */
    constructor(line: number, column: number, baseType: TypeNode, readonly size: number) {
        super(line, column, baseType);
    }

    /**
     * AST node representing the type of the elements in the array.
     */
    get baseTypeNode(): TypeNode {
        return this.childNodes[0] as TypeNode;
    }

    override indentedString(_: string): string {
        return `${this.baseTypeNode.toString()}[${this.size}]`;
    }
    

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitArrayTypeNode(this, context);
    }
}