import { TypeNode } from "./index";
import { Visitor } from "./../Visitor";

export type ChannelTypeName = "intchan" | "boolchan" | "stringchan"

/**
 * AST node representing a channel type.
 * @example intchan
 */
export class ChannelTypeNode extends TypeNode {
    readonly nodeName = "ChannelTypeNode";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param channelTypeName The name of the channel type, i.e., "intchan", "boolchan" or "stringchan".
     * @param capacity The capacity of the channel or *null* if the capacity is not specified.
     */
    constructor(line: number, column: number, readonly channelTypeName: ChannelTypeName, readonly capacity: number | null) {
        super(line, column);
    }

    override indentedString(_: string): string {
        const capacityString = (this.capacity) ? this.capacity.toString() : "";
        return `${this.channelTypeName}${capacityString}`;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitChannelTypeNode(this, context);
    }
}