import { TypeNode } from "./index";
import { Visitor } from "./../Visitor";

/**
 * AST node representing a monitor or struct when declaring a variable or argument.
 * @example M
 */
export class ClassTypeNode extends TypeNode {
    readonly nodeName = "ClassTypeNode";
    
    /**
     * 
    * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param className The name of the monitor or struct.
     */
    constructor(line: number, column: number, readonly className: string) {
        super(line, column);
    }

    override indentedString(_: string): string {
        return this.className;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitClassTypeNode(this, context);
    }
}