import { TypeNode } from "./index";
import { Visitor } from "./../Visitor";

export type SimpleTypeName = "void" | "bool" | "int" | "string" | "lock" | "agent"

/**
 * AST node representing the simple pseuCo types.
 * @example void
 * bool
 * int
 * string
 * lock
 * agent
 */
export class SimpleTypeNode extends TypeNode {
    readonly nodeName = "SimpleTypeName";
    
    /**
     * 
     * @param line The (first) line number in which `this` node appears in the pseuCo program.
     * @param column The first column number at which `this` node appears in the pseuCo program.
     * @param typeName The name of the type, i.e., "void", "bool", "int", "string", "lock" or "agent".
     */
    constructor(line: number, column: number, readonly typeName: SimpleTypeName) {
        super(line, column);
    }

    override indentedString(_: string): string {
        return this.typeName;
    }

    override accept<T>(visitor: Visitor<T>, context?: unknown): T {
        return visitor.visitSimpleTypeNode(this, context);
    }
}