import {expect, assert} from "chai";
import fs from "fs";
import {Program, parse, PeggySyntaxError } from "../dist/index";

function removeSingleLineComments(program: string): string {
  const regexp = /\/\/.*/g;
  return program.replace(regexp, "");
}

// Does not work for nested comments! If you need support for nested comments up to nesting level n, add regexps for each nesting up to level n.
function removeMultiLineComments(program: string): string {
  const regexp = /\/\*[^]*?\*\//g;
  return program.replace(regexp, "");
}

function removeComments(program: string): string {
  return removeMultiLineComments(removeSingleLineComments(program));
}


describe("parse and print", function() {
  const allFiles = fs.readdirSync("tests/pseuco-tests/tests/");
  for (const fileName of allFiles) {
    if (/\.pseuco$/.test(fileName)) {
      it(`file ${fileName}`, function() {
        const pseucoCode = fs.readFileSync(`tests/pseuco-tests/tests/${fileName}`).toString();
        const ast = parse(pseucoCode);
        const astString = ast.toString();
        expect(astString.replace(/[\s()]/g,"")).to.be.equal(removeComments(pseucoCode).replace(/[\s()]/g,""));
      });
    }
  }
});

